<?php
//base class for running the pubmed citations plugin

class pubmed_citations {

	//properties
	protected $loader;
	protected $admin;
  	protected $plugin_name;
  	protected $version;

	function __construct(){
     if ( defined( 'PLUGIN_NAME_VERSION' ) ) {
        $this->version = PLUGIN_NAME_VERSION;
      } else {
        $this->version = '1.0.0';
      }
      $this->plugin_name = 'PubMed_Citations';

      $this->load_dependencies();

      add_shortcode('pubmed-citations', array($this,'query_pubmed'));
      $this->loader->add_action('admin_menu',$this->admin,'pubmed_cite_options_page');
      $this->loader->add_action('admin_init',$this->admin,'pubmed_cite_register_settings');
	}


	private function load_dependencies() {

      /**
       * The class responsible for orchestrating the actions and filters of the
       * core plugin.
       */
      require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/pubmed-cite-loader.php';


      /**
       * The class responsible for defining internationalization functionality
       * of the plugin.
       * nsh - I'm skipping this
       */
      //require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-plugin-name-i18n.php';


      /**
       * The class responsible for defining all actions that occur in the admin area.
       */
     require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/pubmed-cite-admin.php';


      /**
       * The class responsible for defining all actions that occur in the public-facing
       * side of the site.
       * nsh - skipping this too
       */
      //require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-plugin-name-public.php';


      $this->loader = new PubMed_Citations_Loader();
      $this->admin = new PubMed_Citations_Admin($this->plugin_name,$this->version);
    }

	public function query_pubmed(){
    $content = NULL;
	  $pmid = NULL;
    $random = rand();
    $repo_articles = array();

   //get api config settings
    $data = get_option('pubmed_cite_options');
    $apikey = $data['api_key'];
    $query = array($data['querystring_1'],$data['querystring_2'],$data['querystring_3'],$data['querystring_4'],$data['querystring_5']);
    $repo = array( "pmc"=>$data['pmc'],"pubmed"=>$data['pm'] );
    $intro_text = $data['intro'];

    //unpack query
    $pmquery = NULL;
    foreach($query as $string):
      if(!empty($string)):
        $string = trim($string);
        if(ctype_alnum($string)):
            $pmquery .= urlencode($string) . "+OR+";
        else:
            //adds the [Text Word] qualifier for anything that contains a non-alpha numeric character
            $pmquery .= urlencode($string)."[Text+Word]" . "+OR+";
        endif;
      endif;
    endforeach;

    //trims the trailing 'OR' from the query
    if(substr($pmquery,-4)=="+OR+"):
      $pmquery = substr($pmquery,0,-4);
    endif;

    //begin writing page content
    $content .= "<link href=.css?rnd=Session$random\" rel=\"stylesheet\" type=\"text/css\"/>\n";

    //header paragraph
   $content .= "<h6 id=pub_subheader>".$intro_text."</h6>";


    //search repo for matching article numbers
		foreach($repo as $k=>$v):
       if(!empty($v)):
          $pmdb = $k;

          $pmbase = 'https://eutils.ncbi.nlm.nih.gov/entrez/eutils/';
          $pmurl1 = $pmbase . "esearch.fcgi?db=$pmdb&term=$pmquery&retmode=json&retmax=1000&api_key=$apikey";

          //make sure pubmed responds
         if($this->pubmed_cite_get_http_response_code($pmurl1) != "200"):
            return "<p>Error - can't contact PubMed at this time</p>";
          else:
            $pmresponse = file_get_contents($pmurl1);
            $pmresults = json_decode($pmresponse,true);
          endif;

          //parse array of found result to get only article numbers
          //this is how pubmed searching works: the initial search returns an array of found article #s
          //you then have to feed the article #s into a subsequent search to get article information
          if(!empty($pmresults)):
            if(!empty($pmresults['esearchresult']['idlist'])):
              $pmArticlesReturned = $pmresults['esearchresult']['idlist'];
              $pmArticlesReturned = array_unique($pmArticlesReturned);
             endif;
          endif;

          //foreach article number, create a query string to feed back to repo
          $pmArticleQuery = NULL;
          if(!empty($pmArticlesReturned)):
            foreach ($pmArticlesReturned as $pmid): //load each article number in a comma delimited query string
              $pmArticleQuery .= $pmid.",";
            endforeach;
            $pmArticleQuery = substr($pmArticleQuery,0,-1);
          endif;

          //query repo for details on all found article numbers
          $pmbase = 'https://eutils.ncbi.nlm.nih.gov/entrez/eutils/';
          $pmurl1 = $pmbase . "esummary.fcgi?db=$pmdb&id=$pmArticleQuery&retmode=json&retmax=1000&usehistory=y&api_key=$apikey";

          //again, ensure pubmed responds
          if($this->pubmed_cite_get_http_response_code($pmurl1) != "200"):
            return "<p>Error - can't contact PubMed at this time</p>";
          else:
            $pmresponse = file_get_contents($pmurl1);
            $pmresults = json_decode($pmresponse,true);
          endif;

          $results[] = array($pmresults);

      endif;
		endforeach;

    //merge results from pubmed and pmc into one big array
		foreach($results as $array):
      $repo_articles = array_merge($repo_articles,$array);
		endforeach;

    //create a separate array of UIDS and PM IDs to separate duplicates
    $found_articles = array();
		$counter = count($repo_articles);
		for($c=0;$c<$counter;$c++):
		  //get list of uids from repo array
		  if(!empty($repo_articles[$c]['result']['uids'])):
		    $uids = $repo_articles[$c]['result']['uids'];
		  endif;
		  foreach($uids as $id):
		    $articleids = $repo_articles[$c]['result'][$id]['articleids'];
		    foreach($articleids as $article):
          if($article['idtype']=='pmid'):
            $pmid = $article['value'];
          endif;
		    endforeach;
		    $found_articles[$id] = ($pmid);
		  endforeach;
		endfor;

    $found_articles = array_unique($found_articles);

    //for each UID left in $found_articles, get the corresponding article data from $repo_articles
    //and append to $content
    $counter = 1;
    $size = count($repo_articles);
    if(!empty($found_articles)):
      foreach($found_articles as $articleid=>$pmid):
        $title = $this->pubmed_cite_scan_repo_arr($size,$repo_articles,"result,$articleid,title","title");
        $journal = $this->pubmed_cite_scan_repo_arr($size,$repo_articles,"result,$articleid,fulljournalname","fulljournalname");
        $volume = $this->pubmed_cite_scan_repo_arr($size,$repo_articles,"result,$articleid,volume","volume");
        $issue = $this->pubmed_cite_scan_repo_arr($size,$repo_articles,"result,$articleid,issue","issue");
        $pages = $this->pubmed_cite_scan_repo_arr($size,$repo_articles,"result,$articleid,pages","pages");
        $pmcid = $this->pubmed_cite_scan_repo_arr($size,$repo_articles,"result,$articleid,articleids","pmcid");
        $doival = $this->pubmed_cite_scan_repo_arr($size,$repo_articles,"result,$articleid,articleids","doi");

        if(!empty($this->pubmed_cite_scan_repo_arr($size,$repo_articles,"result,$articleid,articleids","doi"))):
          $doi = 'doi';
        endif;

        $authors = $this->pubmed_cite_scan_repo_arr($size,$repo_articles,"result,$articleid,authors","authors");

        if(!empty($pmcid)):
          $url = "https://www.ncbi.nlm.nih.gov/pmc/articles/$pmcid";
        else:
          $url = "https://pubmed.ncbi.nlm.nih.gov/articles/$articleid";
        endif;

        $pubdates = $this->pubmed_cite_scan_repo_arr($size,$repo_articles,"result,$articleid,pubdate","pubdate");

        if(!empty($url) AND !empty($authors) AND !empty($title)):
          if(empty($journal)): $journal = NULL; endif;
          if(empty($doi)): $doi = NULL; endif;
          if(empty($doival)): $doival = NULL; endif;
          if(empty($pmcid)): $pmcid = $articleid; endif;
          if(empty($volume)): $volume = NULL; endif;
          if(empty($pages)): $pages = NULL; endif;
          if(empty($issue)): $issue = 0; endif;
          $content .= "<p class=pmarticle id=$pmid aid=$articleid >$counter.) <a href=$url target=_blank>$authors ($pubdates). $title. $journal, $volume($issue), $pages. $doi:$doival. $pmcid</a></p>";
        endif;
        $counter++;
      endforeach;
    endif;

    return $content;
}

private function pubmed_cite_scan_repo_arr($count,$arr,$path,$field){
  $value = NULL;
  $path_arr = explode(",",$path);

  if(count($path_arr)>3){
    //assumes count of path_arr will always be 3
    die;
  } else {
    for($x=0;$x<$count;$x++){
      if(!empty($arr[$x][$path_arr[0]][$path_arr[1]][$path_arr[2]])){
        if(str_contains($path,"articleids")){
            if(!empty($arr[$x][$path_arr[0]][$path_arr[1]][$path_arr[2]])){
              $art_arr = $arr[$x][$path_arr[0]][$path_arr[1]][$path_arr[2]];
              $l = count($art_arr);
              for($x=0;$x<$l;$x++){
                if($art_arr[$x]['idtype']==$field){
                  if(!empty($art_arr[$x]['value'])){
                    $value = $art_arr[$x]['value'];
                   }
                }
              }
            }
        } elseif(str_contains($path,"authors")){
          if(!empty($arr[$x][$path_arr[0]][$path_arr[1]][$path_arr[2]])){
            $auth_arr = $arr[$x][$path_arr[0]][$path_arr[1]][$path_arr[2]];
              foreach($auth_arr as $author){
                if($author['authtype']=="Author"){
                  $value .= $author['name'] . ", ";
                }
              }
            }
          $value = substr($value,0,-2);
        } else {
          if(empty($value)){
            $value = $arr[$x][$path_arr[0]][$path_arr[1]][$path_arr[2]];
          }
        }
      }
    }
    return $value;

  }
}

private function pubmed_cite_get_http_response_code($url) {
		$headers = get_headers($url);
		return substr($headers[0], 9, 3);
	}

public function run() {
  $this->loader->run();
}


///testing below - all copied from admin class



}

?>
